/* (C) Copyright,  David H. Bennett, 1992 */

/* Program : RUC - RemoteAccess User Command Line Editor
 * David Bennett / 12/13/91 / Ver 1.1
 *
 * This program will allow you make changes to a Hudson USERS.BBS record via the
 * command line or batch file.
 *
 * RUC.C
 *
 * Errorlevels
 * 
 *  0 ok
 *  1 could not find user
 *  2 could not open USERS.BBS
 *  3 command syntax error (Usage is displayed)
 *
 * History:
 *   1.0 12/13/91 - Original version.
 *   1.1 2/3/92   - First release added '$' variable value and -c option for
 *     checking if a user exists.
 *
 */

#include <bios.h>
#include <dir.h>
#include <dos.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include "pascal.h"
#include "ruc.h"
#include "getopt.h"

char *usage = \
	"Program : RUC - RemoteAccess User Command Line Editor\n" \
    "David Bennett / 12/13/91 / Ver 1.0\n" \
    "\n" \
    "This program will allow you make changes to a USERS.BBS record via the\n" \
    "command line or batch file.\n" \
    "\n" \
	"Usage RUC -lb -p D:\\PATH -u User_Name -v Var{=NewValue} {-v ...}\n" \
    "\n" \
    "-l = listing (no write)  -p = USERS.BBS path           -b = bare mode\n" \
    "-u = User's name         -v = Value to display|update  -c = User Search\n" \
    "\n" \
    "Var is : aflags=-X?         fsmsged=0,1                nokill=0,1\n" \
    "         ansi=0,1           fsmsgview=0,1              password=str\n" \
    "         avt=0,1            group=num                  pending=num\n" \
    "         bflags=-X?         hidden=0,1                 phone=str\n" \
    "         cflags=-X?         hotkeys=0,1                quiet=0,1\n" \
    "         clrscr=0,1         lastdate=##-##-##          screenlength=num\n" \
    "         credit=num         lastpwdchange=##-##-##     security=num\n" \
    "         dataphone=str      lastread=num               todayk=num\n" \
    "         deleted=0,1        lasttime=##:##             uploads=num\n" \
    "         dflags=-X?         location=str               uploadsk=num\n" \
    "         downloads=num      moreprompt=0,1             xferprior=0,1\n" \
    "         downloadsk=num     msgsposted=num\n" \
    "         elapsed=num        nocalls=num\n";

char *usage2 = \
    "\n" \
    "You may use a value of \"$\" as a variable value to query for user\n" \
    "input (or stdin input).\n" \
    "\n" \
    "Examples:\n" \
    "\n" \
    "    RUC -l -uDavid_Bennett -vsecurity       (Displays users security settings)\n" \
    "    RUC -uFrank_Kennedy -vaflags=?X??????   (Turns on user flag A2)\n" \
    "    RUC -b -uJudy_Caronia -vansi            (Outputs just a 0 or 1 to stdout)\n" \
    "    RUC -c -uJoe_Taibi                      (If Joe Taibi exists, ERRORLEVEL=0)\n" \
    "    RUC -uMike_Reardon -vcredit=$           (Input mail credit at console)\n" \
    "\n";

char *vtokens[] = { "location", "password", "dataphone", "phone", "lasttime",
					"lastdate", "aflags", "bflags", "cflags", "dflags", "credit",
					"pending", "msgsposted", "lastread", "security", "uploads",
                    "downloads", "uploadsk", "downloadsk", "todayk", "elapsed",
					"screenlength", "lastpwdchange", "group", "deleted",
                    "clrscr", "moreprompt", "ansi", "nokill", "xferprior",
                    "quiet", "hotkeys", "avt", "fsmsgview", "hidden", 
                    "nocalls", "fsmsged", ""
};

/***********************************
 ******* UTILITY FUNCTIONS *********
 ***********************************/

/*
 * Strip underscores and replace them with spaces.
 */
char *stripuscore(char *st)
{
    char *p=st;
    while (*p) { if (*p == '_') *p = ' ';  p++; }
    return(st);
}

/*
 * Parse token and new value from argument.  Returns token.
 */
char *parsetokval(char *tok, char *val, char *arg)
{
    char *p=arg, *t=tok, *v=val;

    while (*p && (*p != '=')) *t++ = *p++;
    *t = '\0';
    if (*p == '=') p++;
    while (*p) *v++ = *p++;
    *v = '\0';
    stripuscore(val);
    return(tok);
}

/*
 * Return token number given token string.  Returns token number or
 * -1 on error.
 */
int tokenval(char *tok, char *vtok[])
{
	int i=0;
	char **p=vtok;

	while (*p[i]) {
		if (!stricmp(tok,p[i])) return(i);
		i++;
	}
	return(-1);
}

/*
 * Convert a flags string into a binary value applying the current value.
 */
unsigned char flagconv(unsigned char oldval, char *flagstr)
{
	unsigned char newval=oldval;
	int pos;
	char *p=flagstr;

	pos=1;
	do {
		if (strchr("yYxX1+", *p)) newval |= (unsigned char)pos;
		if (strchr("nN-0", *p)) newval &= ((unsigned char)pos ^ 0xFF);
		p++;
		pos <<= 1;
	} while (*p && (pos < 0xFF));
	return(newval);
}

/*
 * Return a string representing a binary value in X- format
 */
char *bytetoxdash(char *xdash, unsigned char val)
{
    char *p=xdash;
    int pos;

    pos=1;
    do {
        if (val & pos) *p = 'X';
        else *p = '-';
        p++;
        pos <<= 1;
    } while (pos < 0xFF);
    *p = '\0';
    return(xdash);
}

/********************************
 ******* FILE FUNCTIONS *********
 ********************************/

/*
 * Read a record from USERS.BBS.  Returns offset at begining of record
 * for writing this.  Returns a -1 on error.
 */
long readuser(FILE *fil, userrec *ur)
{
    long fileoff;
    
    fileoff = ftell(fil);

    if (!fread(ur, sizeof(userrec), 1, fil)) return(-1);

    passtr_to_c(ur->name);
    passtr_to_c(ur->location);
    passtr_to_c(ur->password);
    passtr_to_c(ur->dataphone);
    passtr_to_c(ur->phone);
    passtr_to_c(ur->lasttime);
    passtr_to_c(ur->lastdate);

    return(fileoff);
}

/*
 * Write a record to USERS.BBS at 'offset' returns -1 on error.
 */
int writeuser(FILE *fil, userrec *ur, long offset)
{
    fseek(fil, offset, SEEK_SET);

    cstr_to_pas(ur->name);
    cstr_to_pas(ur->location);
    cstr_to_pas(ur->password);
    cstr_to_pas(ur->dataphone);
    cstr_to_pas(ur->phone);
    cstr_to_pas(ur->lasttime);
    cstr_to_pas(ur->lastdate);

    if (!fwrite(ur, sizeof(userrec), 1, fil)) return(-1);
    else return(0);
}

/*******************************
 ******* OUTPUT HELPERS ********
 *******************************/

/*
 * Show a current value
 */
void fpcurval(FILE *fil, char *tok, int bare, char *fmt, ...)
{
	char str[128];
    va_list argptr;

	va_start( argptr, fmt );		/* Initialize va_ functions */
	vsprintf( str, fmt, argptr );	/* prints string to buffer  */
	va_end(argptr);
    if (bare) 
    	fprintf(fil, "%s\n", str);
    else
    	fprintf(fil, "Current %-14s : %s\n", tok, str);
}

/*
 * Show new value
 */
void fpnewval(FILE *fil, char *tok, int bare, char *fmt, ...)
{
	char str[128];
    va_list argptr;

	va_start( argptr, fmt );		/* Initialize va_ functions */
	vsprintf( str, fmt, argptr );	/* prints string to buffer  */
	va_end(argptr);
    if (bare)
        fprintf(fil, "%s\n", str);
    else
        fprintf(fil, "New %-18s : %s\n", tok, str);
}

void showusage(void)
{
    fprintf(stderr, usage);
    fprintf(stderr, "Press spacebar...");
    bioskey(0);
    fprintf(stderr, "\r                 ");
    fprintf(stderr, usage2);
    exit(0);
}

void main(int argc, char **argv)
{
    static userrec urec;
	static char path[MAXPATH]="USERS.BBS";
	static char username[128]="";
	static char *opts="p:P:u:U:v:V:lLbBcC";
	static char token[128], val[128], xdash[9];
    char ch, list=0, bare=0, checkmode=0;
    FILE *userfil;
    long uoffset;
    int i;

    if (argc == 1) showusage();

    /*
     * First parameter pass to get USERS.BBS path and name to search.
     */
    while ((ch = getopt(argc, argv, opts)) != EOF) {

        switch (tolower(ch)) {
            case 'p' :
                strcpy(path, optarg);
                if (path[strlen(path)-1] != '\\') strcat(path, "\\");
                strcat(path, "USERS.BBS");
				break;

			case 'u' :
				strcpy(username, optarg);
				stripuscore(username);
                break;

            case 'l' :
                list=1;
                break;

            case 'b' :
                bare=1;
                break;

            case 'c' :
                checkmode=1;
                break;

            case '?' :
                showusage();
                break;
        }
    }

#ifdef DEBUG
	fprintf(stderr, "* Path is           : %s\n", path);
    fprintf(stderr, "* User to update is : %s\n", username);
    fprintf(stderr, "* List mode is      : %s\n", (list ? "ON" : "OFF"));
#endif

    /* 
     * Try and open USERS.BBS
     */
#ifdef DEBUG
	fprintf(stderr, "- Opening USERS.BBS...\n");
#endif
    userfil = fopen(path, "r+b");
    if (!userfil) {
		fprintf(stderr, "! Could not open USERS.BBS!\n");
        fcloseall();
		exit(2);
	}
	setmode(fileno(userfil), O_DENYNONE);

    /*
     * Try and find the user
     */
#ifdef DEBUG
    fprintf(stderr, "- Searching for user...\n");
#endif
    while (!feof(userfil)) {

        uoffset = readuser(userfil, &urec);
		if (!stricmp(urec.name, username)) break;
        if (feof(userfil)) {
            if (!checkmode)
                fprintf(stderr, "! Cound not find user record.\n");
			fcloseall();
            exit(1);
        }
    }

    if (checkmode) {
        fcloseall();
        exit(0);     /* User was found */
    }

    /*
     * Second command line pass, parse command and update record.
     */
	optind = 1;
    while ((ch = getopt(argc, argv, opts)) != EOF) {

        if (tolower(ch) != 'v') continue;

		parsetokval(token, val, optarg);
		i = tokenval(token, vtokens);

		if (i == -1) {
            fprintf(stderr, "! Invalid token in argument : %s\n", optarg);
            fcloseall();
            exit(1);
        }

        /* 
         * Check for user input token
         */
        if (!strcmp(val, "$")) {
            printf("Please input new value for [%s] :", token);
            gets(val);
        }

#ifdef DEBUG
        fprintf(stderr, "User : %s\n", urec.name);
#endif
        switch (i) {

            case ULOCATION:
                fpcurval(stdout, token, bare, "%s", urec.location);
                if (list || !(*val)) break;
                memcpy(urec.location, val, sizeof(urec.location));
                urec.location[sizeof(urec.location)-1] = '\0';
                fpnewval(stdout, token, bare, "%s", urec.location);
                break;
            case UPASSWORD:
                fpcurval(stdout, token, bare, "%s", urec.password);
                if (list || !(*val)) break;
                memcpy(urec.password, val, sizeof(urec.password));
                urec.password[sizeof(urec.password)-1] = '\0';
                fpnewval(stdout, token, bare, "%s", urec.password);
                break;
            case UDATAPHONE:
                fpcurval(stdout, token, bare, "%s", urec.dataphone);
                if (list || !(*val)) break;
                memcpy(urec.dataphone, val, sizeof(urec.dataphone));
                urec.dataphone[sizeof(urec.dataphone)-1] = '\0';
                fpnewval(stdout, token, bare, "%s", urec.dataphone);
                break;
            case UPHONE:
                fpcurval(stdout, token, bare, "%s", urec.phone);
                if (list || !(*val)) break;
                memcpy(urec.phone, val, sizeof(urec.phone));
                urec.phone[sizeof(urec.phone)-1] = '\0';
                fpnewval(stdout, token, bare, "%s", urec.phone);
                break;
            case ULASTTIME:
                fpcurval(stdout, token, bare, "%s", urec.lasttime);
                if (list || !(*val)) break;
                memcpy(urec.lasttime, val, sizeof(urec.lasttime));
                urec.lasttime[sizeof(urec.lasttime)-1] = '\0';
                fpnewval(stdout, token, bare, "%s", urec.lasttime);
                break;
            case ULASTDATE:
                fpcurval(stdout, token, bare, "%s", urec.lastdate);
                if (list || !(*val)) break;
                memcpy(urec.lastdate, val, sizeof(urec.lastdate));
                urec.lastdate[sizeof(urec.lastdate)-1] = '\0';
                fpnewval(stdout, token, bare, "%s", urec.lastdate);
                break;
			case UAFLAGS:
				fpcurval(stdout, token, bare, "%s", bytetoxdash(xdash, urec.aflags));
				if (list || !(*val)) break;
				urec.aflags = flagconv(urec.aflags, val);
				fpnewval(stdout, token, bare, "%s", bytetoxdash(xdash, urec.aflags));
				break;
			case UBFLAGS:
				fpcurval(stdout, token, bare, "%s", bytetoxdash(xdash, urec.bflags));
				if (list || !(*val)) break;
				urec.bflags = flagconv(urec.bflags, val);
				fpnewval(stdout, token, bare, "%s", bytetoxdash(xdash, urec.bflags));
				break;
			case UCFLAGS:
				fpcurval(stdout, token, bare, "%s", bytetoxdash(xdash, urec.cflags));
				if (list || !(*val)) break;
				urec.cflags = flagconv(urec.cflags, val);
				fpnewval(stdout, token, bare, "%s", bytetoxdash(xdash, urec.cflags));
				break;
			case UDFLAGS:
				fpcurval(stdout, token, bare, "%s", bytetoxdash(xdash, urec.dflags));
				if (list || !(*val)) break;
				urec.dflags = flagconv(urec.dflags, val);
				fpnewval(stdout, token, bare, "%s", bytetoxdash(xdash, urec.dflags));
				break;
			case UCREDIT:
				fpcurval(stdout, token, bare, "%u", urec.credit);
				if (list || !(*val)) break;
				urec.credit = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.credit);
				break;
			case UPENDING:
				fpcurval(stdout, token, bare, "%u", urec.pending);
				if (list || !(*val)) break;
				urec.pending = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.pending);
				break;
			case UMSGSPOSTED:
				fpcurval(stdout, token, bare, "%u", urec.msgsposted);
				if (list || !(*val)) break;
				urec.msgsposted = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.msgsposted);
				break;
			case ULASTREAD:
				fpcurval(stdout, token, bare, "%u", urec.lastread);
				if (list || !(*val)) break;
				urec.lastread = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.lastread);
				break;
			case USECURITY:
				fpcurval(stdout, token, bare, "%u", urec.security);
				if (list || !(*val)) break;
				urec.security = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.security);
				break;
			case UUPLOADS:
				fpcurval(stdout, token, bare, "%u", urec.uploads);
				if (list || !(*val)) break;
				urec.uploads = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.uploads);
				break;
			case UDOWNLOADS:
				fpcurval(stdout, token, bare, "%u", urec.downloads);
				if (list || !(*val)) break;
				urec.downloads = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.downloads);
				break;
			case UUPLOADSK:
				fpcurval(stdout, token, bare, "%u", urec.uploadsk);
				if (list || !(*val)) break;
				urec.uploadsk = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.uploadsk);
				break;
			case UDOWNLOADSK:
				fpcurval(stdout, token, bare, "%u", urec.downloadsk);
				if (list || !(*val)) break;
				urec.downloadsk = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.downloadsk);
				break;
			case UTODAYK:
				fpcurval(stdout, token, bare, "%d", urec.todayk);
				if (list || !(*val)) break;
				urec.todayk = atoi(val);
				fpnewval(stdout, token, bare, "%d", urec.todayk);
				break;
			case UELAPSED:
				fpcurval(stdout, token, bare, "%d", urec.elapsed);
				if (list || !(*val)) break;
				urec.elapsed = atoi(val);
				fpnewval(stdout, token, bare, "%d", urec.elapsed);
				break;
			case USCREENLENGTH:
				fpcurval(stdout, token, bare, "%u", urec.screenlength);
				if (list || !(*val)) break;
				urec.screenlength = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.screenlength);
				break;
			case ULASTPWDCHANGE:
				fpcurval(stdout, token, bare, "%u", urec.lastpwdchange);
				if (list || !(*val)) break;
				urec.lastpwdchange = (unsigned char)atoi(val);
				fpnewval(stdout, token, bare, "%u", urec.lastpwdchange);
				break;
			case UGROUP:
				fpcurval(stdout, token, bare, "%u", urec.group);
				if (list || !(*val)) break;
				urec.group = (unsigned char)atoi(val);
				fpnewval(stdout, token, bare, "%u", urec.group);
				break;
			case UDELETED:
				fpcurval(stdout, token, bare, "%c", (urec.attr.deleted ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr.deleted = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr.deleted ? 'Y' : 'N'));
				break;
			case UCLRSCR:
				fpcurval(stdout, token, bare, "%c", (urec.attr.clrscr ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr.clrscr = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr.clrscr ? 'Y' : 'N'));
				break;
			case UMOREPROMPT:
				fpcurval(stdout, token, bare, "%c", (urec.attr.moreprompt ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr.moreprompt = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr.moreprompt ? 'Y' : 'N'));
				break;
			case UANSI:
				fpcurval(stdout, token, bare, "%c", (urec.attr.ansi ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr.ansi = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr.ansi ? 'Y' : 'N'));
				break;
			case UNOKILL:
				fpcurval(stdout, token, bare, "%c", (urec.attr.nokill ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr.nokill = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr.nokill ? 'Y' : 'N'));
				break;
			case UXFERPRIOR:
				fpcurval(stdout, token, bare, "%c", (urec.attr.xferprior ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr.xferprior = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr.xferprior ? 'Y' : 'N'));
				break;
			case UQUIET:
				fpcurval(stdout, token, bare, "%c", (urec.attr.quiet ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr.quiet = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr.quiet ? 'Y' : 'N'));
				break;
			case UHOTKEYS:
				fpcurval(stdout, token, bare, "%c", (urec.attr2.hotkeys ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr2.hotkeys = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr2.hotkeys ? 'Y' : 'N'));
				break;
			case UAVT:
				fpcurval(stdout, token, bare, "%c", (urec.attr2.avt ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr2.avt = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr2.avt ? 'Y' : 'N'));
				break;
			case UFSMSGVIEW:
				fpcurval(stdout, token, bare, "%c", (urec.attr2.fsmsgview ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr2.fsmsgview = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr2.fsmsgview ? 'Y' : 'N'));
				break;
			case UHIDDEN:
				fpcurval(stdout, token, bare, "%c", (urec.attr2.hidden ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr2.hidden = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr2.hidden ? 'Y' : 'N'));
				break;
			case UNOCALLS:
				fpcurval(stdout, token, bare, "%u", urec.nocalls);
				if (list || !(*val)) break;
				urec.nocalls = (unsigned int)atol(val);
				fpnewval(stdout, token, bare, "%u", urec.nocalls);
				break;
			case UFSMSGED:
				fpcurval(stdout, token, bare, "%c", (urec.attr.fsmsged ? 'Y' : 'N'));
				if (list || !(*val)) break;
				urec.attr.fsmsged = (strchr("yYxX1+", *val) ? 1 : 0);
				fpnewval(stdout, token, bare, "%c", (urec.attr.fsmsged ? 'Y' : 'N'));
				break;
		}
	}

    /*
     * Updating the user record.
     */
    if (!list) {
#ifdef DEBUG
        fprintf(stderr, "- Updating USERS BBS.\n");
#endif
        writeuser(userfil, &urec, uoffset);
    }

    fcloseall();
}

